Index: deliver.h
===================================================================
--- deliver.h	(revision 49)
+++ deliver.h	(working copy)
@@ -30,4 +30,13 @@
 /* Returns a new unique Message-ID */
 const char *deliver_get_new_message_id(void);
 
+/* MTA/MDA-related tweaks */
+typedef enum {
+	MDA_Invalid = -1
+	, MDA_Default = 0
+	, MDA_Sendmail = 1	/* Bounces are send via MTA
+							and report back error via stdout */
+} mdaMode_t;
+extern mdaMode_t mdaMode;
+
 #endif
Index: mail-send.c
===================================================================
--- mail-send.c	(revision 49)
+++ mail-send.c	(working copy)
@@ -37,6 +37,19 @@
     int ret;
 
     orig_msgid = mail_get_first_header(mail, "Message-ID");
+
+    if(mdaMode == MDA_Sendmail) {
+    	/* Let sendmail handle the Bounce and pass it the reason via stdout */
+
+	    printf("msgid=%s rejected reason: %s",
+		   orig_msgid == NULL ? "" : str_sanitize(orig_msgid, 80),
+		   str_sanitize(reason, 512));
+	    i_info("mode sendmail: msgid=%s: rejection reason: %s",
+		   orig_msgid == NULL ? "" : str_sanitize(orig_msgid, 80),
+		   str_sanitize(reason, 512));
+	    return -1;	/* cause Tempfail */
+    }
+
     return_addr = deliver_get_return_address(mail);
     if (return_addr == NULL) {
 	    i_info("msgid=%s: Return-Path missing, rejection reason: %s",
Index: deliver.c
===================================================================
--- deliver.c	(revision 49)
+++ deliver.c	(working copy)
@@ -51,6 +51,7 @@
 static const char *default_mailbox_name = NULL;
 static bool tried_default_save = FALSE;
 static bool no_mailbox_autocreate = FALSE;
+mdaMode_t mdaMode = MDA_Invalid;
 
 static struct module *modules;
 static struct ioloop *ioloop;
@@ -449,9 +450,20 @@
 {
 	printf(
 "Usage: deliver [-c <config file>] [-d <destination user>] [-m <mailbox>]\n"
-"               [-n] [-f <envelope sender>]\n");
+"               [-n] [-f <envelope sender>] [--mode <operation mode>]\n"
+"    operation mode: may be one of: default, sendmail\n"
+);
 }
 
+static mdaMode_t mdaMode2code(const char str[]) {
+	if(strcmp(str, "default") == 0) {
+		return MDA_Default;
+	} else if(strcmp(str, "sendmail") == 0) {
+		return MDA_Sendmail;
+	}
+	return MDA_Invalid;
+}
+
 int main(int argc, char *argv[])
 {
 	const char *config_path = DEFAULT_CONFIG_FILE;
@@ -533,6 +545,19 @@
 					       "Missing envelope argument");
 			}
 			envelope_sender = argv[i];
+		} else if (strcmp(argv[i], "--mode") == 0) {
+			/* MDA mode */
+			i++;
+			if (i == argc) {
+				i_fatal_status(EX_USAGE,
+					       "Missing MDA mode argument");
+			}
+			mdaMode = mdaMode2code(argv[i]);
+			if(mdaMode < MDA_Default) {
+				print_help();
+				i_fatal_status(EX_USAGE,
+						   "Unknown MDA argument: %s", argv[i]);
+			}
 		} else if (argv[i][0] != '\0') {
 			print_help();
 			i_fatal_status(EX_USAGE,
@@ -582,6 +607,21 @@
 					  TRUE, version);
 	}
 
+	if(mdaMode < MDA_Default) {
+		if(getenv("LDA_MODE") != NULL) {
+			const char *mode = getenv("LDA_MODE");
+			mdaMode = mdaMode2code(mode);
+			if(mdaMode < MDA_Default) {
+				print_help();
+				i_fatal_status(EX_USAGE,
+						   "Unknown MDA argument: %s", mode);
+			}
+		} else {
+			mdaMode = MDA_Default;	/* No mode set, default */
+		}
+	}
+
+
 	if (destination != NULL) {
 		auth_socket = getenv("AUTH_SOCKET_PATH");
 		if (auth_socket == NULL)
